# frozen_string_literal: true

module Izokatu
  # Namespace for OpenSSL classes
  module Openssl
    # List of OpenSSL private key ciphers
    PKEY_CIPHERS = OpenSSL::Cipher.ciphers.map(&:upcase)

    # Wrap cipher are not working in Openssl. Here text of error:
    # OpenSSL::Cipher::CipherError: wrap mode not allowed
    # Or straight from openssl 1.1.1d
    # error:0607B0AA:digital envelope routines:EVP_CipherInit_ex:wrap mode not allowed:../crypto/evp/evp_enc.c:161
    PKEY_CIPHERS.delete_if { |name| name.include?('WRAP') }

    # List of OpenSSL public key EC ciphers
    PBKEY_EC_CIPHERS = OpenSSL::PKey::EC.builtin_curves.map! { |cipher_name, _| cipher_name }
  end
end
