# frozen_string_literal: true

module Izokatu
  # Abstract class for decrypters
  #
  # @abstract Subclasses are containing implementation of {#decrypt_data!}
  class Decrypter
    extend Callable

    include Contracts

    # @return decrypter instance
    attr_reader :decrypter
    # @return [String] encrypted data for decryption
    attr_reader :encrypted_data
    # @return [Hash] decrypted data for export
    attr_reader :decrypted_data

    Contract Contracts::HashOf[Symbol, String] => Any
    # Initializing option for decryption
    #
    # @param encrypted_data (#encrypted_data)
    #
    # @since 0.1.0
    def initialize(encrypted_data:)
      import_encrypted_data!(encrypted_data)
    end

    Contract String => String
    # Importing encrypted data from param
    #
    # @param encrypted_data (#encrypted_data)
    #
    # @return [String] (#encrypted_data)
    #
    # @since 0.1.0
    def import_encrypted_data!(encrypted_data)
      @encrypted_data = encrypted_data
    end

    Contract None => Contracts::HashOf[Symbol, String]
    # Performing data decryption
    #
    # @return [Hash] decrypted data
    #
    # @since 0.1.0
    def perform
      decrypt_data!
    end

    private

    # Decrypting data
    #
    # @raise RuntimeError
    #
    # @since 0.1.0
    def decrypt_data!
      raise 'Not implemented!'
    end
  end
end
