# frozen_string_literal: true

module Izokatu
  # Abstract class for encrypters
  #
  # @abstract Subclasses are containing implementation of {#encrypt_data!}
  class Encrypter
    extend Callable

    include Contracts

    # @return encrypter instance
    attr_reader :encrypter
    # @return [String] clear data for encryption
    attr_reader :clear_data
    # @return [Hash] encrypted data for export
    attr_reader :encrypted_data

    # Default options for Izokatu::Encrypter
    DEFAULT_OPTIONS = {
      clear_data_string: 'Testing, testing...'
    }.freeze

    Contract Contracts::HashOf[Symbol, String] => Any
    # Initializing option for encryption
    #
    # @param clear_data (#clear_data)
    #
    # @since 0.1.0
    def initialize(clear_data:)
      import_clear_data!(clear_data)
    end

    Contract String => String
    # Imporing clear data from options
    #
    # @param clear_data (#clear_data)
    #
    # @return [String] (#clear_data)
    #
    # @since 0.1.0
    def import_clear_data!(clear_data)
      @clear_data = clear_data || DEFAULT_OPTIONS[:clear_data_string]
    end

    Contract None => Contracts::ArrayOf[Contracts::HashOf[Symbol, Maybe[String]]]
    # Performing encryption
    #
    # @return [Hash] encrypted data with params
    #
    # @since 0.1.0
    def perform
      encrypt_data!
    end

    private

    # Encrypting data`
    #
    # @raise RuntimeError
    #
    # @since 0.1.0
    def encrypt_data!
      raise 'Not implemented!'
    end
  end
end
