# frozen_string_literal: true

module Izokatu
  # Abstract class for exporters
  #
  # @abstract Subclasses are containing implementation of {#perform}
  class Exporter
    extend Callable

    include Contracts
    include Izokatu::Helpers

    # @return [String] data for export
    attr_reader :data

    Contract Contracts::HashOf[Symbol, Or[Contracts::HashOf[Symbol, Maybe[String]], Bool]] => Any
    # Initializing data for export
    #
    # @param data (#data)
    # @param encode [TrueClass || FalseClass] Enable/disable encoding of exported data
    #
    # @since 0.1.0
    def initialize(data:, encode:)
      @data = encode ? encode_data(data) : data
    end

    # Performing export of initialized data
    #
    # @raise RuntimeError
    #
    # @since 0.1.0
    def perform
      raise 'Not implemented!'
    end
  end
end
