# frozen_string_literal: true

module Izokatu
  # Izokatu exporter for exporting data in some file
  class FileExporter < Izokatu::Exporter
    # @return [String] name of file to export
    attr_reader :filename

    Contract Contracts::HashOf[Symbol, Or[Contracts::HashOf[Symbol, Maybe[String]], String, Bool]] => Any
    # Initializing data for file export
    #
    # @param data (#data)
    # @param filename (#filename)
    # @param encode [TrueClass || FalseClass] Enable/disable encoding of exported data
    #
    # @since 0.1.0
    def initialize(data:, filename:, encode:)
      super(data: data, encode: encode)
      @filename = filename
    end

    Contract None => nil
    # Performing export of initialized data
    #
    # @since 0.1.0
    def perform
      File.write(filename, JSON.generate(data))
      nil
    end
  end
end
