# frozen_string_literal: true

module Izokatu
  # Izokatu importer for importing data from some file
  class FileImporter
    extend Callable

    include Contracts
    include Izokatu::Helpers

    # @return [String] name of file to import from
    attr_reader :filename
    # @return [TrueClass, FalseClass] status of ability to delete file after import
    attr_reader :delete_imported
    # @return [TrueClass || FalseClass] status of ability to decode imported data
    attr_reader :decode

    Contract Contracts::HashOf[Symbol, Or[String, Bool]] => Any
    # Initializing options for import
    #
    # @param filename (#filename)
    # @param delete_imported (#delete_imported)
    # @param decode (#decode)
    #
    # @since 0.1.0
    def initialize(filename:, delete_imported:, decode:)
      @filename = filename
      @delete_imported = delete_imported
      @decode = decode
    end

    Contract None => Contracts::HashOf[Symbol, Or[Maybe[String]]]
    # Performing import of data
    #
    # @return [Hash] imported data
    #
    # @since 0.1.0
    def perform
      raise 'No data file!' unless File.exist?(filename)

      encoded = File.read(filename)
      File.delete(filename) if delete_imported
      data = JSON.parse(encoded).transform_keys(&:to_sym)
      decode ? decode_data(data) : data
    end
  end
end
