# frozen_string_literal: true

module Izokatu
  # Izokatu importer for importing data from some function, via :data param
  class FunctionImporter
    extend Callable

    include Contracts
    include Izokatu::Helpers

    # @return [String] data for import
    attr_reader :data

    Contract Contracts::HashOf[Symbol, Or[Contracts::HashOf[Symbol, Maybe[String]], String, Bool]] =>
             Or[Contracts::HashOf[Symbol, Maybe[String]], String]
    # Initializing options for import
    #
    # @param data (#data)
    # @param decode [TrueClass || FalseClass] Enable/disable decoding of imported data
    #
    # @since 0.1.0
    def initialize(data:, decode:)
      @data = decode ? decode_data(data) : data
    end

    Contract None => Or[Contracts::HashOf[Symbol, Maybe[String]], String]
    # Performing import of data
    #
    # @return [Hash] imported data
    #
    # @since 0.1.0
    def perform
      data
    end
  end
end
