# frozen_string_literal: true

module Izokatu
  module Openssl
    module PrivateKey
      module Auth
        module CCM
          # OpenSSL private key encrypter for authenticated ciphers in CCM mode
          class Encrypter < Izokatu::Openssl::PrivateKey::Auth::Encrypter
            # Default iv (nonce) length for ciphers in CCM mode
            DEFAULT_CCM_IV_LENGTH = 7
            # Default authentication tag length for ciphers in CCM mode
            DEFAULT_CCM_AUTH_TAG_LENGTH = 8

            private

            Contract None => Any
            # Initializing encrypter params
            #
            # @since 0.1.0
            def initialize_encrypter_params!
              initialize_auth_ccm_encrypter_params!
              super
            end

            Contract None => Any
            # Set length for iv and auth_tag before their initialization
            #
            # @since 0.1.0
            def initialize_auth_ccm_encrypter_params!
              encrypter.iv_len = DEFAULT_CCM_IV_LENGTH
              encrypter.auth_tag_len = DEFAULT_CCM_AUTH_TAG_LENGTH
            end

            Contract Maybe[String] => Any
            # Initializing encrypter auth params
            #
            # @since 0.1.0
            def initialize_auth_encrypter_params!(auth_data)
              initialize_static_message_length!
              super
            end

            Contract None => Any
            # Initializing message length for cipher in CCM mode
            #
            # @since 0.1.0
            def initialize_static_message_length!
              encrypter.ccm_data_len = clear_data.length unless cipher.include?('ARIA')
            end
          end
        end
      end
    end
  end
end
