# frozen_string_literal: true

module Izokatu
  # Namespace for RbNaCl classes
  module Rbnacl
    # Abstract class for Rbnacl decrypters
    #
    # @abstract Subclasses are containing implementation of {#create_decrypter!}
    class Decrypter < Izokatu::Decrypter
      # @return [String] initialization vector for one-time use
      attr_reader :nonce

      # Classes of RbNaCl keys
      RBNACL_KEY_CLASSES = [
        RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PrivateKey,
        RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PublicKey
      ].freeze

      Contract Contracts::HashOf[Symbol, String] => Any
      # Initializing option for decryption
      #
      # @param encrypted_data (#encrypted_data)
      # @param nonce (#nonce)
      #
      # @since 0.1.0
      def initialize(encrypted_data:, nonce:)
        super(encrypted_data: encrypted_data)
        @nonce = nonce
        create_decrypter!
      end

      # Creating decrypter instance
      #
      # @raise RuntimeError
      #
      # @since 0.1.0
      def create_decrypter!
        raise 'Not implemented!'
      end
    end
  end
end
