# frozen_string_literal: true

module Izokatu
  # Namespace for RbNaCl classes
  module Rbnacl
    # Abstract class for Rbnacl encrypters
    #
    # @abstract Subclasses are containing implementation of {#create_encrypter!}
    class Encrypter < Izokatu::Encrypter
      # @return [String] initialization vector for one-time use
      attr_reader :nonce

      Contract Contracts::HashOf[Symbol, String] => Any
      # Initializing option for encryption
      #
      # @param clear_data (#clear_data)
      #
      # @since 0.1.0
      def initialize(clear_data:)
        super
        create_encrypter!
        generate_nonce!
      end

      # Creating encrypter instance
      #
      # @raise RuntimeError
      #
      # @since 0.1.0
      def create_encrypter!
        raise 'Not implemented!'
      end

      Contract None => String
      # Generating nonce from encrypter instance
      #
      # @return [String] initialization vector for one-time use
      #
      # @since 0.1.0
      def generate_nonce!
        @nonce = RbNaCl::Random.random_bytes(encrypter.nonce_bytes)
      end
    end
  end
end
