# frozen_string_literal: true

module Izokatu
  module Rbnacl
    # Namespace for RbNaCl private key classes
    module PrivateKey
      # RbNaCl private key decrypter
      class Decrypter < Izokatu::Rbnacl::Decrypter
        # @return [String] key for private key encryption/decryption
        attr_reader :key
        # @return [String] authenticated data
        attr_reader :auth_data

        # Default options for Izokatu::Rbnacl::Decrypter
        DEFAULT_OPTIONS = {
          auth_data: ''
        }.freeze

        Contract Contracts::HashOf[Symbol, Or[Or[*RBNACL_KEY_CLASSES], Maybe[String]]] => Any
        # Initializing option for decryption
        #
        # @param encrypted_data (#encrypted_data)
        # @param nonce (#nonce)
        # @param key (#key)
        # @param auth_data (#auth_data)
        #
        # @since 0.1.0
        def initialize(key:, encrypted_data:, nonce:, auth_data:)
          @key = key
          super(encrypted_data: encrypted_data, nonce: nonce)
          @auth_data = auth_data || DEFAULT_OPTIONS[:auth_data]
        end

        Contract None => RbNaCl::AEAD::XChaCha20Poly1305IETF
        # Creating decrypter instance
        #
        # @return [RbNaCl::AEAD::XChaCha20Poly1305IETF] decrypter instance
        #
        # @since 0.1.0
        def create_decrypter!
          @decrypter = RbNaCl::AEAD::XChaCha20Poly1305IETF.new(key)
        end

        Contract None => Contracts::HashOf[Symbol, String]
        # Decrypting data
        #
        # @return [Hash] decrypted data
        #
        # @since 0.1.0
        def decrypt_data!
          { decrypted_data_string: decrypter.decrypt(nonce, encrypted_data, auth_data) }
        end
      end
    end
  end
end
