# frozen_string_literal: true

module Izokatu
  module Rbnacl
    module PrivateKey
      # RbNaCl private key encrypter
      class Encrypter < Izokatu::Rbnacl::Encrypter
        # @return [String] key for private key encryption/decryption
        attr_reader :key
        # @return [String] authenticated data
        attr_reader :auth_data

        Contract Contracts::HashOf[Symbol, Maybe[String]] => Any
        # Initializing option for encryption
        #
        # @param clear_data (#clear_data)
        # @param auth_data (#auth_data)
        #
        # @since 0.1.0
        def initialize(clear_data:, auth_data:)
          generate_key!
          super(clear_data: clear_data)
          @auth_data = auth_data
        end

        Contract None => String
        # Generating key
        #
        # @since 0.1.0
        def generate_key!
          @key = RbNaCl::Random.random_bytes(RbNaCl::SecretBox.key_bytes)
        end

        Contract None => RbNaCl::AEAD::XChaCha20Poly1305IETF
        # Generating encrypter instance from key
        #
        # @since 0.1.0
        def create_encrypter!
          @encrypter = RbNaCl::AEAD::XChaCha20Poly1305IETF.new(key)
        end

        Contract None => Contracts::ArrayOf[Contracts::HashOf[Symbol, Maybe[String]]]
        # Encrypting data
        #
        # @return (Array) encrypted data with decrypter params
        #
        # @since 0.1.0
        def encrypt_data!
          [
            { encrypted_data_string: encrypter.encrypt(nonce, clear_data, auth_data) },
            decrypter_params
          ]
        end

        Contract None => Contracts::HashOf[Symbol, Maybe[String]]
        # Returning decrypter params
        #
        # @return [Hash] decrypter params
        #
        # @since 0.1.1
        def decrypter_params
          {
            nonce: nonce,
            key: key,
            auth_data: auth_data
          }
        end
      end
    end
  end
end
