# frozen_string_literal: true

module Izokatu
  module Rbnacl
    # Namespace for RbNaCl public key classes
    module PublicKey
      # RbNaCl public key decrypter
      class Decrypter < Izokatu::Rbnacl::Decrypter
        # @return [RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PublicKey] public key
        attr_reader :public_key
        # @return [RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PrivateKey] private key
        attr_reader :private_key

        Contract Contracts::HashOf[Symbol, Or[Or[*RBNACL_KEY_CLASSES], String]] => Any
        # Initializing option for decryption
        #
        # @param encrypted_data (#encrypted_data)
        # @param nonce (#nonce)
        # @param public_key (#public_key)
        # @param private_key (#private_key)
        #
        # @since 0.1.0
        def initialize(public_key:, private_key:, encrypted_data:, nonce:)
          @public_key = public_key
          @private_key = private_key
          super(encrypted_data: encrypted_data, nonce: nonce)
        end

        Contract None => RbNaCl::Boxes::Curve25519XSalsa20Poly1305
        # Initializing decrypter instance
        #
        # @return [RbNaCl::Boxes::Curve25519XSalsa20Poly1305] decrypter instance
        #
        # @since 0.1.0
        def create_decrypter!
          @decrypter = RbNaCl::Box.new(public_key, private_key)
        end

        Contract None => Contracts::HashOf[Symbol, String]
        # Decrypting data
        #
        # @return [Hash] decrypted data
        #
        # @since 0.1.0
        def decrypt_data!
          { decrypted_data_string: decrypter.decrypt(nonce, encrypted_data) }
        end
      end
    end
  end
end
