# frozen_string_literal: true

module Izokatu
  module Rbnacl
    module PublicKey
      # RbNaCl keys generator
      class KeysGenerator < Izokatu::KeysGenerator
        private

        Contract None => RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PrivateKey
        # Performing generation of RbNaCl private key
        #
        # @return [RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PrivateKey] RbNaCl private key
        #
        # @since 0.1.0
        def generate_private_key
          RbNaCl::PrivateKey.generate
        end

        Contract RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PrivateKey =>
                 RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PublicKey
        # Performing generation of RbNaCl public key
        #
        # @return [RbNaCl::Boxes::Curve25519XSalsa20Poly1305::PublicKey] RbNaCl public key
        #
        # @since 0.1.0
        def generate_public_key(private_key)
          private_key.public_key
        end
      end
    end
  end
end
